import junit.framework.TestCase;

/*
 * Created on 2004. 2. 25.
 *  
 */
/**
 * @author Jisung, Ahn
 *  
 */
public class KWICTestCase extends TestCase {
	final String LF = "\n";
	public void testOneWord() {
		String input = "abc";
		String expected = "abc";
		SortedShifts shifts = new SortedShifts(input);
		assertEquals(expected, shifts.nextShift());
		assertEquals(null, shifts.nextShift());
	}
	public void testTwoWords() {
		String input = "abc def";
		String[] expected = new String[]{"abc def", "def abc"};
		SortedShifts shifts = new SortedShifts(input);
		assertEquals(expected[0], shifts.nextShift());
		assertEquals(expected[1], shifts.nextShift());
		assertEquals(null, shifts.nextShift());
	}
	public void testThreeWords() {
		String input = "hij klm opq";
		String[] expected = new String[]{"hij klm opq", "klm opq hij", "opq hij klm"};
		SortedShifts shifts = new SortedShifts(input);
		assertEquals(expected[0], shifts.nextShift());
		assertEquals(expected[1], shifts.nextShift());
		assertEquals(expected[2], shifts.nextShift());
		assertEquals(null, shifts.nextShift());
	}
	public void testLines() {
				
				String input = "abc"+LF+"def";
				String[] expected = new String[] { "abc", "def"};
				SortedShifts shifts = new SortedShifts(input);
				assertEquals(expected[0], shifts.nextShift());
				assertEquals(expected[1], shifts.nextShift());
				assertEquals(null, shifts.nextShift());
	}
	
	public void testComplex() {
		String input = "abc def"+LF+"hij klm opq";
		String[] expected = new String[] { 
				"abc def", "def abc",
				"hij klm opq", "klm opq hij", "opq hij klm"
		};
		SortedShifts shifts = new SortedShifts(input);
		for(int i=0; i < expected.length;i++){
			assertEquals(expected[i], shifts.nextShift());
		}
		assertEquals(null, shifts.nextShift());
	}
	
	public void testSorted() {
		String input = "def abc"+LF+"klm opq hij";
		String[] expected = new String[] { 
				"abc def", "def abc",
				"hij klm opq", "klm opq hij", "opq hij klm"
		};
		SortedShifts shifts = new SortedShifts(input);
		for(int i=0; i < expected.length;i++){
			assertEquals(expected[i], shifts.nextShift());
		}
		assertEquals(null, shifts.nextShift());
	}
	
	public void testHasNext() {
		String input = "abc def"+LF+"hij klm opq";
		String[] expected = new String[] { 
				"abc def", "def abc",
				"hij klm opq", "klm opq hij", "opq hij klm"
		};
		SortedShifts shifts = new SortedShifts(input);
		for(int i=0; i < expected.length;i++){
			assertTrue(shifts.hasNext());
			assertNotNull(shifts.nextShift());
		}
		assertFalse("No more shifts",shifts.hasNext());
		assertNull("No more shifts",shifts.nextShift());
		
	}
	
	public void testSplit() {
		String src = "abc def";
		String[] splited = SortedShifts.splitWords(src);
		String[] expected = new String[]{"abc", "def"};
		for (int i = 0; i < 2; i++) {
			assertEquals(expected[i], splited[i]);
		}
		src = "abc";
		splited = SortedShifts.splitWords(src);
		assertEquals(1, splited.length);
		assertEquals(src, splited[0]);
		src = "hij klm opq";
		splited = SortedShifts.splitWords(src);
		assertEquals(3, splited.length);
		assertEquals("hij", splited[0]);
		assertEquals("klm", splited[1]);
		assertEquals("opq", splited[2]);
	}
}
