/*
 * Created on 2004. 3. 3
 *
 */
package net.narusas.facet;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

/**
 * @author jsahn
 *
 */
public class Resource implements Comparable {

	protected Map _topics = new HashMap();

	protected static HashSet ID_SET = new HashSet();
	protected static long NEXT_ID = 0;

	protected String _id;
	protected String _name;
	
	 

	/**
	 * @param string
	 */
	public Resource(String name) {
		this(Resource.nextId(), name);
	}

	/**
	 * @param object
	 * @param name
	 */
	public Resource(String id, String name) {
		_name = name;
		if (Resource.existID(id)) {
			_id = Resource.nextId();
		} else {
			_id = id;
		}
		Facet.NAME_FACET.createTopic(_name);

	}

	/**
	 * @param id
	 * @return
	 */
	private static boolean existID(String id) {
		return ID_SET.contains(id);
	}

	/**
	 * 
	 */
	private synchronized static String nextId() {
		String nextId = null;
		while (true) {
			nextId = String.valueOf(NEXT_ID++);
			if (!Resource.existID(nextId)) {
				break;
			}
		}
		return nextId;
	}

	/**
	 * @return
	 */
	public String getId() {
		return _id;
	}

	public String getName() {
		return _name;
	}

	/**
	 * @param topic
	 */
	public void addTopic(Topic topic) {
		topic.addResource(this);
		
		_topics.put(topic.getFacet(), topic);
	}

	/**
	 * 
	 */
	public Topic[] getTopics() {
		return _topics.size() == 0
			? null
			: (Topic[]) _topics.values().toArray(new Topic[0]);
	}

	/**
	 * @param topic2
	 */
	public void removeTopic(Topic topic) {
		topic.removeResource(this);
		_topics.remove(topic.getFacet());
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object target) {
		Resource t = (Resource) target;
		return _id.compareTo(t.getId());
	}

	public String toString() {
		return _id;
	}

	public Topic[] matchFacet(Facet facet) {
		if ("Name".equals(facet.getId())){
			return _name;
		}
		return (Topic) _topics.get(facet);
	}

	public void dispose() {
		Topic[] topics = getTopics();
		for (int i = 0; i < topics.length; i++) {
			topics[i].removeResource(this);
		}
		_topics.clear();
		_topics = null;
	}
}
