/*
 * Created on 2004. 3. 3
 *
 */
package net.narusas.facet;

import junit.framework.TestCase;

/**
 * @author jsahn
 *
 */
public class TestFacet extends TestCase {

	private Topic topic3;
	private Resource res3;
	private Resource res2;
	private Topic topic2;
	private Topic topic1;
	private Resource res1;
	private Facet facet1;
	private Facet facet2;

	/**
	 * Constructor for TestFacet.
	 * @param arg0
	 */
	public TestFacet(String arg0) {
		super(arg0);
	}

	/* (non-Javadoc)
		 * @see junit.framework.TestCase#setUp()
		 */
	protected void setUp() throws Exception {
		facet1 = new Facet("kind1");
		facet2 = new Facet("kind2");
		res1 = new Resource("name1");
		res2 = new Resource("name2");
		res3 = new Resource("name3");
		topic1 = facet1.createTopic("topic1");
		topic2 = facet2.createTopic("topic2");
		topic3 = facet2.createTopic("topic2");
		super.setUp();
	}

	/* (non-Javadoc)
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		facet1 = null;
		res1 = null;
		topic1 = null;
		topic2 = null;
		super.tearDown();
	}

	public void testResource() {
		assertEquals("name1", res1.getName());
	}

	public void testFacetAndTopic() {
		assertEquals("kind1", facet1.getName());

		assertEquals("topic1", topic1.getName());
		assertEquals(topic1, facet1.getTopic("topic1"));
		assertNull(facet1.getTopic("noneTopic"));

		Topic childTopic = facet1.createTopic("childTopic");
		topic1.addChild(childTopic);
		assertEquals(1, topic1.getChilds().length);
		assertEquals(topic1, childTopic.getParent());

	}

	public void testResourceAndTopic() {

		res1.addTopic(topic1);
		assertEquals(1, res1.getTopics().length);
		assertEquals(topic1, res1.getTopics()[0]);
		assertEquals(1, topic1.getResources().length);

		res1.addTopic(topic2);
		assertEquals(2, res1.getTopics().length);
		assertEquals(topic1, res1.getTopics()[0]);
		assertEquals(topic2, res1.getTopics()[1]);
		assertEquals(1, topic2.getResources().length);

		res1.removeTopic(topic2);
		assertEquals(1, res1.getTopics().length);
		assertEquals(topic1, res1.getTopics()[0]);
		assertEquals(1, topic1.getResources().length);

		assertNull(topic2.getResources());

	}

	public void testFindResourceBasic() {
		res1.addTopic(topic1);
		res1.addTopic(topic2);

		ResourceMatch classifier = new ResourceMatch(null);
		classifier.addCondition(topic1);
		classifier.addCondition(topic2);
		Resource[] result = classifier.match();
		assertEquals(1, result.length);
		assertEquals(res1, result[0]);
	}

	public void testFindResourceComplex() {
		res1.addTopic(topic1);
		res1.addTopic(topic2);

		Resource res2 = new Resource("name2");
		res2.addTopic(topic1);

		ResourceMatch matcher = new ResourceMatch(null);
		matcher.addCondition(topic1);
		Resource[] result = matcher.match();
		assertEquals(2, result.length);
		assertEquals(res1, result[0]);
		assertEquals(res2, result[1]);
		
		matcher = new ResourceMatch(null);
		matcher.addCondition(topic1);
		matcher.addCondition(topic2);
		result = matcher.match();
		assertEquals(1, result.length);
		assertEquals(res1, result[0]);
	}
	
	public void testFindResourceEx() {
		res1.addTopic(topic1);
		res1.addTopic(topic2);

		Resource res2 = new Resource("name2");
		
		
		
	}

	public void testSortedResource() {
		res1.addTopic(topic1);
		res1.addTopic(topic2);		
		res2.addTopic(topic1);		
		res3.addTopic(topic1);
		ResourceMatch matcher = new ResourceMatch(null);
		matcher.addCondition(topic1);
		Resource[] result = matcher.match();
		assertEquals(3, result.length);
		assertEquals(res1, result[0]);
		assertEquals(res2, result[1]);
		assertEquals(res3, result[2]);
	}
	
	public void testClassifier() {
		FacetClassifier classifier = new FacetClassifier("My Library");
		classifier.addResource(res1);
		classifier.addResource(res2);
		classifier.addResource(res3);		
		assertEquals(3, classifier.getAllResources().length);
		
		classifier.addFacet(facet1);
		assertEquals(1, classifier.getFacets().length);
		assertEquals(facet1, classifier.getFacets()[0]);
		
		res1.addTopic(topic1);
		res1.addTopic(topic2);
		
		res2.addTopic(topic1);
		
		res3.addTopic(topic2);
		
		Resource[] result = classifier.match(new Topic[] {topic1});
		assertEquals(2, result.length);
		assertEquals(res1, result[0]);
		assertEquals(res2, result[1]);
		
		result = classifier.match(new Topic[] {topic2});
		assertEquals(2, result.length);
		assertEquals(res1, result[0]);
		assertEquals(res3, result[1]);
		
		result = classifier.match(new Topic[] {topic1, topic2});
		assertEquals(1, result.length);
		assertEquals(res1, result[0]);		
	}
	
	public void testChlidResource() {
		topic1.addChild(topic2);
		res1.addTopic(topic2);
		
		assertEquals(1, topic1.getResources().length);
		assertEquals(res1, topic1.getResources()[0]);
	}
	
	public void testChildFacet() {
		topic1.addChild(topic2);
		topic1.addChild(topic3);
		
		assertEquals(1, facet1.getRootTopic().length);
	}
	
	public void testResourceRemove() {
		res1.addTopic(topic1);
		res1.addTopic(topic2);
		
		assertEquals(1, topic1.getResources().length);
		assertEquals(1, topic2.getResources().length);
		
		assertEquals(res1, topic1.getResources()[0]);
		assertEquals(res1, topic2.getResources()[0]);
		
		res1.dispose();
		
		assertNull(topic1.getResources());
		assertNull(topic2.getResources());
		
	}
	
	public void testMatchFacet() {
		
	}

}
