/*
 * Created on 2004. 3. 3
 *
 */
package net.narusas.facet;

import java.util.*;

/**
 * @author jsahn
 *
 */
public class Topic extends Observable {

	private Facet _facet;
	private Map _resources = new HashMap();
	;

	private Topic _parent;

	private Map _childs = new HashMap();

	private String _id;
	private String _name;

	private static HashSet ID_SET = new HashSet();
	private static long NEXT_ID;
	

	/**
	 * @param name
	 */
	public Topic(String name, Facet facet) {
		this(Topic.nextId(), name, facet);
	}

	public Topic(String id, String name, Facet facet) {
		_facet  = facet;
		_name = name;
		
		
		if (Topic.existID(id)) {
			_id = Topic.nextId();
		} else {
			_id = id;
		}
	}
	
	public Facet getFacet() {
		return _facet;
	}

	/**
			 * @param id
			 * @return
			 */
	private static boolean existID(String id) {
		return ID_SET.contains(id);
	}

	/**
	 * 
	 */
	private synchronized static String nextId() {
		String nextId = null;
		while (true) {
			nextId = String.valueOf(NEXT_ID++);
			if (!Topic.existID(nextId)) {
				break;
			}
		}
		return nextId;
	}

	/**
	 * @return
	 */
	public String getId() {
		return _id;
	}

	public String getName() {
		return _name;
	}

	/**
	 * @param childTopic
	 */
	public void addChild(Topic childTopic) {
		childTopic.setParent(this);
		_childs.put(childTopic.getId(), childTopic);
		notifyObservers(childTopic);
	}

	/**
	 * 
	 */
	public Topic[] getChilds() {
		return _childs.size() == 0
			? null
			: (Topic[]) _childs.values().toArray(new Topic[0]);
	}

	/**
	 * @return
	 */
	public Topic getParent() {
		return _parent;
	}

	void setParent(Topic parent) {
		_parent = parent;
		notifyObservers(parent);
	}

	/**
	 * 
	 */
	public Resource[] getResources() {
		HashSet list = new HashSet();

		Topic[] childs = getChilds();
		if (childs != null) {
			for (int i = 0; i < childs.length; i++) {
				Utils.addAll(list, childs[i].getResources());
			}
		}
		list.addAll(_resources.values());

		return list.size() == 0
			? null
			: (Resource[]) list.toArray(new Resource[0]);
	}

	/**
	 * @param resource
	 */
	public void addResource(Resource resource) {
		_resources.put(resource.getId(), resource);
		notifyObservers(resource);
	}

	/**
	 * @param resource
	 */
	public void removeResource(Resource resource) {
		_resources.remove(resource.getId());
		notifyObservers(resource);
	}

	/**
	 * @return
	 */
	public Map getResourceMap() {
		return _resources;
	}

	public String toString() {
		return _name;
	}

}
